
###
###  Copyright 2000-2008 University of Illinois Board of Trustees
###  All rights reserved.
###
###  etc_exports.pm - psgconf action class for generating /etc/exports
###
###  Campus Information Technologies and Educational Services
###  University of Illinois at Urbana-Champaign
###

package PSGConf::Action::GenerateFile::etc_exports;

use strict;

use PSGConf::Action::GenerateFile;

our @ISA = qw(PSGConf::Action::GenerateFile);

###############################################################################
###  generate() method
###############################################################################

sub generate
{
    my ( $self, $fh, $psgconf ) = @_;
    my ( $path, $opt, @options, %hosts, $exp, $root_option );
    my ( $access_label, $opt_start, $opt_sep, $host_sep, $opts );

    $exp = $self->{exports};

    if ( $psgconf->data_obj('platform')->match('freebsd') )
    {

        # Format of /etc/exports is:
        # $PATH [ options ] [ list of hosts who have access ]
        $access_label = '';
        $opt_start    = ' -';
        $opt_sep      = ' ';
        $host_sep     = ' ';
        $root_option  = 'maproot=root:wheel';
    } elsif ( $psgconf->data_obj('platform')->match('linux') )
    {

        # TODO: Add Linux support and remove the BSD_exports.pm Action.
        # Format of /etc/exports is:
        # $PATH [hostname1](options) [ hostname2(options) ... ]
        $root_option = 'no_root_squash';
    } else
    {

        # Format of /etc/exports (AIX) is:
        # $PATH [ options ] [ -access=list of hosts ]
        $access_label = ' -access=';
        $opt_start    = ',';
        $opt_sep      = '=';
        $host_sep     = ':';
    }

    foreach $path ( sort keys %$exp )
    {
        @options = ();
        undef %hosts;

        foreach $opt ( keys %{ $exp->{$path} } )
        {
            if ( !defined( $exp->{$path}->{$opt} ) )
            {
                push( @options, $opt );
            } elsif ( !ref( $exp->{$path}->{$opt} ) )
            {
                push( @options, $opt . '=' . $exp->{$path}->{$opt} );
            } elsif ( ref( $exp->{$path}->{$opt} ) eq 'HASH' )
            {
                map { $hosts{$_} = undef; } keys %{ $exp->{$path}->{$opt} }
                  if ( $opt =~ m/^(ro|rw|root)$/ );

                next
                  if ( $opt eq 'ro' );

                next
                  if ( $opt eq 'rw'
                    && $psgconf->data_obj('platform')->match('freebsd') );

                push( @options,
                        $opt 
                      . $opt_sep
                      . join( ':', sort keys %{ $exp->{$path}->{$opt} } ) );
            }
        }

        $opts = ' -' . join( $opt_start, @options );

        $opts .= $access_label . join( $host_sep, sort keys %hosts )
          if ( !exists( $exp->{$path}->{access} )
            && %hosts );

        print $fh $path . $opts . "\n";
    }

    return 1;
}

###############################################################################
###  documentation
###############################################################################

1;

__END__

=head1 NAME

PSGConf::Action::GenerateFile::etc_exports - action to generate /etc/exports

=head1 SYNOPSIS

  use PSGConf::Action::GenerateFile::etc_exports;

  $psgconf->register_actions(
		PSGConf::Action::GenerateFile::etc_exports->new(
			'name'		=> '/etc/exports',
			'exports'	=> {
				"/path/to/export" => {
					ro	=> { host1, host2, ... },
					rw	=> { host1, host2, ... },
					root	=> { host1, host2, ... },
					anon	=> -1,
					secure	=> undef,
					...
				},
				...
			},
			...
		),
		...
	);

=head1 DESCRIPTION

The B<PSGConf::Action::GenerateFile::etc_exports> module provides a
B<PSGConf> action class for generating the F</etc/exports> file.

The B<PSGConf::Action::GenerateFile::etc_exports> class is derived from
the B<PSGConf::Action::GenerateFile> class, but it defines/overrides
the following methods:

=over 4

=item generate()

Generates the F</etc/exports> file based on the value of the
I<exports> attribute.

=back

In addition to the attributes supported by
the B<PSGConf::Action::GenerateFile> class, the
B<PSGConf::Action::GenerateFile::etc_exports> class supports the
following attributes:

=over 4

=item I<exports>

A reference to a hash that maps absolute paths to a hash containing the
options for that path to be exported with.

In the sub-hash containing the export options, the value may be a scalar,
in which case it is encoded as the value of the option, or a reference
to a hash, in which case the keys of this sub-sub-hash will be considered
multiple values for the specified option.

=back

=head1 SEE ALSO

L<perl>

L<PSGConf>

L<PSGConf::Action::GenerateFile>

=cut


