
###
###  Copyright 2000-2008 University of Illinois Board of Trustees
###  All rights reserved.
###
###  PSGConf::Action::GenerateFile::newsyslog - psgconf Action for generating /etc/newsyslog.conf
###
###  Campus Information Technologies and Educational Services
###  University of Illinois at Urbana-Champaign
###

package PSGConf::Action::GenerateFile::newsyslog;

use strict;

use PSGConf::Action::GenerateFile;
use PSGConf::Util;

our @ISA = qw(PSGConf::Action::GenerateFile);

my @ColumnMap = ( 'mode', 'count', 'size', 'period', 'flags' );

my %FlagsMap = (
    'compress'     => 'J',
    'post_command' => '!N'
);

my %DefaultsMap = (
    'mode'   => 644,
    'count'  => '%{logfiles_count}',
    'size'   => '*',
    'period' => 'daily',
    'flags'  => '-'
);

my %PeriodMap = (
    'daily'   => '$D0',
    'weekly'  => '$W6D0',
    'monthly' => '$M1D0',
    'yearly'  => '@01T05'
);

sub _find_field
{
    my ( $self, $psgconf, $field, $file ) = @_;
    my ( $file2, $res );

    $file2 = &PSGConf::Util::_expand_tokens( $psgconf, $file );

    $res = (
        exists $self->{options}->{$file2}->{$field}
        ? $self->{options}->{$file2}->{$field}
        : (
            exists $self->{options}->{$file}->{$field}
            ? $self->{options}->{$file}->{$field}
            : (
                exists $DefaultsMap{$field} ? $DefaultsMap{$field}
                : 'N/A'
            )
        )
    );

    # Translate any special tokens
    $res = &PSGConf::Util::_expand_tokens( $psgconf, $res )
      if ( $res =~ m/%{.*}/ );

    # Map the period back to newsyslog format.
    $res = exists $PeriodMap{$res} ? $PeriodMap{$res} : $res
      if ( $field eq 'period' );

    return $res;
}

###############################################################################
###  generate() method
###############################################################################

sub generate
{
    my ( $self, $fh, $psgconf ) = @_;
    my ( $file, $line, $switch, $opt, $opt2, $flag );

    foreach $file ( @{ $self->{files} } )
    {
        $line = '';
        foreach $switch (@ColumnMap)
        {
            $opt = _find_field( $self, $psgconf, $switch, $file );

            # Augment the flags field with other keys from our options hash
            if ( $switch eq 'flags' )
            {
                foreach $flag ( keys %FlagsMap )
                {
                    $opt2 = _find_field( $self, $psgconf, $flag, $file );

           # If we have a ! in the FlagsMap, then invert the order of the search
           # i.e. we will add the Flag if the field was NOT found.
                    if (   ( $FlagsMap{$flag} =~ /^!/ && $opt2 eq 'N/A' )
                        || ( $FlagsMap{$flag} !~ /^!/ && $opt2 ne 'N/A' ) )
                    {
                        $opt2 = $FlagsMap{$flag};
                        $opt2 =~ s/^!//;
                        $opt = $opt ne '-' ? $opt . $opt2 : $opt2;
                    }
                }
            }

            # Now add the switch to our line
            if ( $opt ne 'N/A' )
            {
                $line .= ' ' if ( length $line && $line !~ /\s$/ );
                $line .= $opt if ( defined $opt );
            }
        }

        $file = &PSGConf::Util::_expand_tokens( $psgconf, $file )
          if ( $file =~ m/%{.*}/ );

        print $fh "$file $line\n";
    }

    return 1;
}

###############################################################################
###  documentation
###############################################################################

1;

__END__

=head1 NAME

PSGConf::Action::GenerateFile::newsyslog - generate /etc/newsyslog.conf file

=head1 SYNOPSIS

  use PSGConf::Action::GenerateFile::newsyslog;

  $psgconf->register_actions(
		PSGConf::Action::GenerateFile::newsyslog->new(
			'name'	=> '/etc/newsyslog.conf',
			'uid'	=> 0,
			'options'	=> { ... },
			'count' => 14,
			'files'	=> [ ... ],
			...
		),
		...
	);

=head1 DESCRIPTION

The B<PSGConf::Action::GenerateFile::newsyslog> module provides a B<PSGConf>
action class for generating the F</etc/newsyslog.conf> file.

The B<PSGConf::Action::GenerateFile::newsyslog> class is derived from the
B<PSGConf::Action::GenerateFile> class, but it defines/overrides the
following methods:

=over 4

=item generate()

Creates the F</etc/newsyslog.conf> file.

=back

In addition to the attributes supported by
the B<PSGConf::Action::GenerateFile> class, the
B<PSGConf::Action::GenerateFile::newsyslog> class supports the following
attributes:

=over 4

=item I<name>

The name of the file to generate, usually F</etc/newsyslog.conf>.

=item I<uid>

What owner the F<newsyslog.conf> file should be.

=item I<options>

Any extra options that are required to be put into F<newsyslog.conf>.

=over 4

=item I<post_command>

If not set, it will add a I<N> to the C<flags> field in F<newsyslog.conf>

=item I<count>

An integer for the number of files to keep.

=item I<mode>

What the mode should be set for the log file.  Defaults to I<644>.

=item I<period>

How often the log files are rotated.  There are several abstracted names:

=over 4

=item I<daily>

Rotate the file every day.  Translated to F<$D0>.

=item I<weekly>

Rotate the file every week.  Translated to F<$W6D0>.

=item I<monthly>

Rotate the file every month.  Translated to F<$M1D0>.

=item I<yearly>

Rotate the file every year.  Translated to F<@01T05>.

=back

=item I<size>

Rotate the log file when it reaches this size.  Defaults to I<*>.

=item I<compress>

If set, it will add a I<J> to the C<flags> field in F<newsyslog.conf>

=item I<flags>

If set, this will be the start of the C<flags> field in F<newsyslog.conf>.
Defaults to I<->.

=back

=item I<count>

The default number of files to keep.

=item I<files>

The list of files we rotate.

=back

=head1 SEE ALSO

C<CE<lt>newsyslog(8)E<gt>>

C<CE<lt>newsyslog.conf(4)E<gt>>

L<perl>

L<PSGConf>

L<PSGConf::Action::GenerateFile>

=cut
