
###
###  Copyright 2000-2004 University of Illinois Board of Trustees
###  All rights reserved.
###
###  PSGConf::Action::PackageManager::RedHat::RPMS - RedHat pkg action type for psgconf
###
###  Campus Information Technologies and Educational Services
###  University of Illinois at Urbana-Champaign
###

package PSGConf::Action::PackageManager::RedHat::RPMS;

use strict;

use RPM qw(evrcmp);
use PSGConf::Action::PackageManager::FTPArchive;
use PSGConf::Util;

our @ISA = qw(PSGConf::Action::PackageManager::FTPArchive);

sub conf
{
    my ( $class, $psgconf ) = @_;

    my ($a) = join( '|', @{ $psgconf->{platform_suffixes} } );
    $a .= "|noarch";

    PSGConf::Action::PackageManager::FTPArchive->conf($psgconf);

    $psgconf->register_data(
        pkg_rpm_cmd => PSGConf::Data::String->new( value => "/bin/rpm" ),
        pkg_regex   => PSGConf::Data::String->new( value => "\.($a)\.rpm" )
    );
}

###############################################################################
###  ComparePkgs() method
###############################################################################
sub ComparePkgs
{
    my ( $self, $psgconf, $pkga, $pkgb ) = @_;
    my ( $pkg, $vera, $verb );

    # Get the package version w/o the name
    $pkg  = $self->Parse($pkga);
    $vera = $pkga;
    $vera =~ s/^\Q${pkg}\E$self->{seperator}->[0]//;

    $pkg  = $self->Parse($pkgb);
    $verb = $pkgb;
    $verb =~ s/^\Q${pkg}\E$self->{seperator}->[0]//;

    # Now call the RPM library comparison operator
    return evrcmp( $vera, $verb );
}

###############################################################################
###  pkg_get_installed() method
###############################################################################

sub pkg_get_installed
{
    my ( $self, $psgconf ) = @_;
    my (%pkg_installed_list);

    return
      if ( !exists( $self->{seperator} ) || ref $self->{seperator} ne 'ARRAY' );

    if ( open( FP, $psgconf->data_obj('pkg_rpm_cmd')->get() . ' -qa 2>&1|' ) )
    {
        while (<FP>)
        {
            chomp;
            my ($pkg) = $self->Parse($_);
            my ($ver) = $_;
            $ver =~ s/^\Q${pkg}\E$self->{seperator}->[0]//;
            push( @{ $pkg_installed_list{$pkg} }, $ver );
        }
        close FP;
    } else
    {
        warn "Could not run "
          . $psgconf->data_obj('pkg_rpm_cmd')->get()
          . " -qa\n";
    }

    return \%pkg_installed_list;
}

###############################################################################
###  pkg_get_latest_version() method
###############################################################################

sub pkg_get_latest_version
{
    my ( $self, $psgconf, $pkg ) = @_;
    my ($regex) =
      "(\Q${pkg}\E\-[^-]+-[^-]+)" . $psgconf->data_obj('pkg_regex')->get();

    return $self->get_latest_version( $psgconf, $pkg,
        $psgconf->data_obj('env_vars')->find('RPM_UPDATE_PATH'), qr{$regex} );
}

###############################################################################
###  pkg_get_dependencies() method
###############################################################################

sub pkg_get_dependencies
{
    my ( $self, $psgconf, $pkg ) = @_;
    my ( @pkgs, $res );
    my ($regex) = "\Q${pkg}\E" . $psgconf->data_obj('pkg_regex')->get();

    ### Determine if the package is locally installed.
    ### Cannot use pkg_installed_list, because it is out of scope :(
    $res = &PSGConf::Util::RunCommand(
        $psgconf->data_obj('pkg_rpm_cmd')->get()
          . ' --quiet -q '
          . $pkg
          . '>/dev/null2>&1',
        1
    );

    if ( !$res )
    {
        @pkgs = $self->_get_local_deps( $psgconf, $pkg );
    } else
    {

        ###
        ### Find the location of the package
        ###
        my (@files) =
          $self->list_files_from_url( $psgconf,
            $psgconf->data_obj('env_vars')->find('RPM_UPDATE_PATH'),
            qr{$regex} );

        @pkgs = $self->_get_remote_deps( $psgconf, $files[0] )
          if ( scalar @files );
    }

    return @pkgs;
}

sub _get_rpm_output
{
    my ( $self, $psgconf, $rpmargs, $pkg ) = @_;
    my ( @lines, $cmd );
    local *FP;

    $cmd =
        $psgconf->data_obj('pkg_rpm_cmd')->get() . ' ' 
      . $rpmargs . ' "' 
      . $pkg . '"';

    if ( open( FP, "$cmd 2>&1 |" ) )
    {
        @lines = <FP>;
        close FP;
    } else
    {
        warn "Could not run ($cmd)\n";
    }

    return @lines;
}

sub _get_local_deps
{
    my ( $self, $psgconf, $pkg ) = @_;

    return $self->_get_deps( $psgconf, '-qR', $pkg );
}

sub _get_remote_deps
{
    my ( $self, $psgconf, $pkg ) = @_;

    return $self->_get_deps( $psgconf, '-qpR', $pkg );
}

### Break this out so we can call it from the RHN module as well.
sub _get_deps
{
    my ( $self, $psgconf, $rpmargs, $pkg ) = @_;
    my ( @pkgs, $line, $line2, $res, $old );

    foreach $line ( sort $self->_get_rpm_output( $psgconf, $rpmargs, $pkg ) )
    {
        chomp $line;

        my (@a) = split /[\s\t]+/o, $line;

        ### If we already looked up this dependency, then skip it....
        next
          if ( $a[0] eq $old );

        $old = $a[0];

        ### Check to see if the line is a package or
        ### a filename by asking rpm again.  If it returns
        ### zero, it is a package.
        if ( scalar @a && length( $a[0] ) )
        {
            $res = &PSGConf::Util::RunCommand(
                $psgconf->data_obj('pkg_rpm_cmd')->get()
                  . ' --quiet -q "'
                  . $a[0]
                  . '"> /dev/null 2>&1',
                1
            );

            if ( !$res )
            {
                push @pkgs, $a[0];
            } elsif ( -f $a[0] )
            {
                foreach $line2 (
                    $self->_get_rpm_output( $psgconf, '-qf', $a[0] ) )
                {
                    chomp $line2;
                    push @pkgs, $line2;
                }

                ###
                ### FIXME: Hack to solve the problem of not finding
                ### rpmlib(XXX) in the rpm database.  Not sure what
                ### package actually provides it.
                ###
            } elsif ( $a[0] !~ /^rpmlib/ )
            {
                foreach $line2 (
                    $self->_get_rpm_output(
                        $psgconf, '-q --whatprovides', $a[0]
                    )
                  )
                {
                    chomp $line2;
                    push @pkgs, $line2;
                }
            }
        }
    }

    return @pkgs;
}

###############################################################################
###  pkg_install() method
###############################################################################

sub pkg_install
{
    my ( $self, $psgconf, $pkg ) = @_;
    my ( @files, $res, $cmd );
    my ($regex) = "\Q${pkg}\E" . $psgconf->data_obj('pkg_regex')->get();

    ###
    ### Find the location of the package
    ###
    @files =
      $self->list_files_from_url( $psgconf,
        $psgconf->data_obj('env_vars')->find('RPM_UPDATE_PATH'), qr{$regex} );

    ### Take the first file and install it.
    if ( scalar(@files) )
    {
        $cmd = $psgconf->data_obj('pkg_rpm_cmd')->get() . ' -U ';
        $cmd .= '--quiet '
          if ( !$psgconf->{verbose} );
        $cmd .= $files[0];

        $res = &PSGConf::Util::RunCommand($cmd);

        print "\n"
          if ( $psgconf->{verbose} );

        return $res;
    }

    return -1;
}

###############################################################################
###  pkg_remove() method
###############################################################################

sub pkg_remove
{
    my ( $self, $psgconf, $pkg, $force ) = @_;
    my ( $res, $cmd );

    $cmd = $psgconf->data_obj('pkg_rpm_cmd')->get() . ' -e --nodeps ';

    $cmd .= '--quiet '
      if ( !$psgconf->{verbose} );
    $cmd .= $pkg;

    $res = &PSGConf::Util::RunCommand($cmd);

    print "\n"
      if ( $psgconf->{verbose} );

    return $res;
}

###############################################################################
###  documentation
###############################################################################

1;

__END__

=head1 NAME

PSGConf::Action::PackageManager::RedHat::RPMS - install RedHat packages

=head1 SYNOPSIS

  use PSGConf::Action::PackageManager::RedHat::RPMS;

  $psgconf->register_actions(
		PSGConf::Action::PackageManager::RedHat::RPMS->new(
			'quiet'		=> 1
		),
		...
	);

=head1 DESCRIPTION

The B<PSGConf::Action::PackageManager::RedHat::RPMS> module provides 
a B<PSGConf> action class for updating RedHat packages from an FTP
Archive.

The B<PSGConf::Action::PackageManager::RedHat::RPMS> class is derived 
from the B<PSGConf::Action::PackageManager::FTPArchive> class, but it 
defines/overrides the following methods:

=over 4

=item I<pkg_get_installed()>

A method that will assign values to the I<pkg_installed_list>, which
will be all the packages currently installed.  The I<pkg_installed_list>
is a reference to a hash of arrays.  The array is sorted to have
versions in ascending order.  Called from the   
B<PSGConf::Action::PackageManager::check()> method.

=item I<pkg_get_latest_version($pkg)>

A method that will return a B<PSGConf::Data::String> that contains
the latest version of $pkg from the FTP repository.  Called from the 
B<PSGConf::Action::PackageManager::check()> method.  It uses the
environment variable B<RPM_UPDATE_PATH> to determine where to look
for these new packages.

=item I<pkg_get_dependencies($pkg)>

A method that will return a B<PSGConf::Data::List> that contains
all the packages that $pkg will depend upon, regardless
if they have been installed or not.   This is for the package on
the FTP repository and $pkg will have the version in it as well.
Called from the B<PSGConf::Action::PackageManager::diff()> method.

=item I<pkg_install($pkg)>

A method that will install a $pkg on the system.  Returns 0 for a
successfull install, non-zero for a failure.  Called from 
B<PSGConf::Action::PackageManager::do()> method.  It uses the
environment variable B<RPM_UPDATE_PATH> to determine where to get
these packages from.

=item I<pkg_remove($pkg, $force)>

A method that will remove $pkg from the system, and may have to force
the removal, depending on the value of $force, which will be a
B<PSGConf::Data::Boolean>.  Returns 0 for a successfull install,
non-zero for a failure.  Called from
B<PSGConf::Action::PackageManager::do()> method.

=back

The constructor also registers the following data objects:

=over 4

=item I<pkg_rpm_cmd>

A B<PSGConf::Data::String> object that determines the location of the
I<rpm> command.  The default is F</bin/rpm>.

=item I<pkg_regex>

A B<PSGConf::Data::String> object that determines what the regex is to
match against the FTPArchive listing.  The default is I<\.$ARCH\.rpm>,
where $ARCH is the output from C<uname -m>.

=back

=head1 SEE ALSO

C<CE<lt>rpm(1)E<gt>>

L<perl>

L<PSGConf>

L<PSGConf::Action>

L<PSGConf::Action::PackageManager::FTPArchive>

L<PSGConf::Util>

=cut

