
###
###  Copyright 2000-2008 University of Illinois Board of Trustees
###  All rights reserved.
###
###  Core.pm - core module for psgconf
###
###  Campus Information Technologies and Educational Services
###  University of Illinois at Urbana-Champaign
###

package PSGConf::Control::Core;

use strict;

use PSGConf::Action::MkDir;
use PSGConf::Action::Remove;
use PSGConf::Action::Symlink;
use PSGConf::Action::CreateFile;
use PSGConf::Data::Hash;
use PSGConf::Data::List;
use PSGConf::Data::String;
use PSGConf::Data::ConfigError;
use PSGConf::Data::FileTest;

###############################################################################
###  decide() method
###############################################################################

sub decide
{
    my ( $self, $psgconf ) = @_;
    my ( $file, $uid, $gid, $mode );

    $mode = PSGConf::Data::Integer->new();

    ### add symlink actions
    foreach $file ( keys %{ $psgconf->data_obj('symlink')->get() } )
    {
        $psgconf->register_actions(
            PSGConf::Action::Symlink->new(
                name    => $file,
                link_to => $psgconf->data_obj('symlink')->find($file)
            )
        );
    }

    ### add mkdir actions
    foreach $file ( keys %{ $psgconf->data_obj('mkdir')->get() } )
    {
        if ( defined $psgconf->data_obj('mkdir')->find($file) )
        {
            $mode->set( $psgconf->data_obj('mkdir')->find($file)->{mode} );
            $uid = $psgconf->data_obj('mkdir')->find($file)->{uid};
            $gid = $psgconf->data_obj('mkdir')->find($file)->{gid};

            $psgconf->register_actions(
                PSGConf::Action::MkDir->new(
                    name => $file,
                    uid  => $uid,
                    gid  => $gid,
                    mode => $mode->get()
                )
            );
        }
    }

    ### add unlink actions
    foreach $file ( @{ $psgconf->data_obj('unlink')->get() } )
    {
        $psgconf->register_actions(
            PSGConf::Action::Remove->new( name => $file ) );
    }

    ### add create_file actions
    foreach $file ( keys %{ $psgconf->data_obj('create_file')->get() } )
    {
        if ( defined $psgconf->data_obj('create_file')->find($file) )
        {
            $mode->set(
                $psgconf->data_obj('create_file')->find($file)->{mode} );
            $uid = $psgconf->data_obj('create_file')->find($file)->{uid};
            $gid = $psgconf->data_obj('create_file')->find($file)->{gid};

            $psgconf->register_actions(
                PSGConf::Action::CreateFile->new(
                    name => $file,
                    uid  => $uid,
                    gid  => $gid,
                    mode => $mode->get()
                )
            );
        }
    }
}

###############################################################################
###  Constructor
###############################################################################

sub new
{
    my ( $class, $psgconf ) = @_;
    my ($self);

    $self = {};
    bless( $self, $class );

    $self->{name} = 'Core';

    $psgconf->register_data(
        etc_inet_dir => PSGConf::Data::String->new(
            'value_abspath' => 1,
            value           => '/etc'
        ),
        hostname => PSGConf::Data::String->new(),
        log_dir  => PSGConf::Data::String->new(
            'value_abspath' => 1,
            value           => '/var/log'
        ),
        pidfile_dir => PSGConf::Data::String->new(
            'value_abspath' => 1,
            value           => '/etc'
        ),
        symlink     => PSGConf::Data::Hash->new( key_abspath    => 1 ),
        mkdir       => PSGConf::Data::Hash->new( value_optional => 1 ),
        unlink      => PSGConf::Data::List->new(),
        create_file => PSGConf::Data::Hash->new( value_optional => 1 ),
        die         => PSGConf::Data::ConfigError->new( fatal   => 1 ),
        warn        => PSGConf::Data::ConfigError->new(),
        filetest    => PSGConf::Data::FileTest->new(),
    );

    return $self;
}

###############################################################################
###  documentation
###############################################################################

1;

__END__

=head1 NAME

PSGConf::Control::Core - psgconf control class for core system identity

=head1 SYNOPSIS

In F<psgconf_modules>:

  Control PSGConf::Control::Core

=head1 DESCRIPTION

The B<PSGConf::Control::Core> module provides a B<psgconf> control object
for configuring core system parameters, such as the system's hostname.
It supports the following methods:

=over 4

=item new()

The constructor.  Its parameter is a reference to the B<PSGConf>
object.  It registers the following data objects:

=over 4

=item I<etc_inet_dir>

A B<PSGConf::Data::String> object that contains the absolute path to the
directory where network-related config files live.  The default is
F</etc>.

=item I<hostname>

A B<PSGConf::Data::String> object that contains the system's hostname.

=item I<log_dir>

A B<PSGConf::Data::String> object that contains the absolute path to the
system's log directory.  The default is F</var/log>.

=item I<pidfile_dir>

A B<PSGConf::Data::String> object that contains the absolute path to the
directory where pid files are stored on the system.  The default is
F</etc>.

=item I<mkdir>

A B<PSGConf::Data::Hash> object whose keys are a list of directories to be
created on the system.  The value is an anomoyous hash that has the keys
of C<uid>, C<gid> and C<mode>.

=item I<unlink>

A B<PSGConf::Data::List> object whose keys are a list of files to be
deleted on the system.

=item I<create_file>

A B<PSGConf::Data::Hash> object whose keys are a list of files to be
created on the system.  The value is an anomoyous hash that has the keys
of C<uid>, C<gid> and C<mode>.

=item I<symlink>

A B<PSGConf::Data::Hash> object that contains a list of symlinks to be
created on the system.  The hash key is the absolute path of the
symlink, and the value is what the link should point to.

=item I<die>

A B<PSGConf::Data::ConfigError> object that, if set, will cause an
immediate configuration processing failure.

=item I<warn>

A B<PSGConf::Data::ConfigError> object that, if set, will cause a
warning message to be emitted.

=item I<filetest>

A B<PSGConf::Data::FileTest> object that will return the perl test
operator value for the given file.

=back

=item decide()

For the paths specified by the I<symlink>, I<unlink>, I<create_file>, and
I<mkdir> data objects, instantiates the appropriate
B<PSGConf::Action::Symlink>, B<PSGConf::Action::Remove>,
B<PSGConf::Action::CreateFile>, and B<PSGConf::Action::MkDir> action
objects.

=back

=head1 SEE ALSO

L<perl>

L<PSGConf>

L<PSGConf::Action::File>

L<PSGConf::Action::MkDir>

L<PSGConf::Action::Symlink>

L<PSGConf::Action::CreateFile>

L<PSGConf::Data::List>

L<PSGConf::Data::Hash>

L<PSGConf::Data::String>

L<PSGConf::Data::ConfigError>

L<PSGConf::Data::FileTest>

L<psgconf-intro>

=cut

